#include <avr/interrupt.h>
#include <stdio.h>
#include <string.h>
#include "led.h"

static const uint8_t dispPin[] = {LED_DISP1_PIN, LED_DISP2_PIN, LED_DISP3_PIN, LED_DISP4_PIN};

static uint8_t dispDigits[4] = {0, 0, 0, 0};

const uint8_t segments[] =
{
	0b00111111,
	0b00000110,
	0b01011011,
	0b01001111,
	0b01100110,
	0b01101101,
	0b01111101,
	0b00000111,
	0b01111111,
	0b01101111,
	0b01000000,
	0b10000000
};


void ledInit() {
	sei();
	TCCR0 = _BV(CS01); // Timer0 CLK/8
	TIMSK = _BV(TOIE0); // przerwanie na przepenienie
	LED_SEG_DDR = 0xff;
	LED_DISP_DDR |= _BV(LED_DISP4_PIN) | _BV(LED_DISP3_PIN) | _BV(LED_DISP2_PIN) | _BV(LED_DISP1_PIN);
}

ISR(TIMER0_OVF_vect)
{
	static uint8_t dispNum = 0;
	LED_DISP_PORT |= _BV(LED_DISP4_PIN) | _BV(LED_DISP3_PIN) | _BV(LED_DISP2_PIN) | _BV(LED_DISP1_PIN);
	LED_DISP_PORT &= ~_BV(dispPin[dispNum]);
	LED_SEG_PORT = ~dispDigits[dispNum];
	dispNum++;
	dispNum %= 4;
}

void ledDispNumber(float num, char format[], uint8_t shift) {
	uint8_t dispNum = 3 - shift;
	char buff[6];
	snprintf(buff, sizeof(buff), format, num);
	buff[sizeof(buff) - 1] = 0;
	for (uint8_t i = 0; i < strlen(buff); i++) {
		char ch = buff[i];
		if (ch >= '0' && ch <= '9') dispDigits[dispNum] = segments[ch - '0'];
		if (ch == '-') dispDigits[dispNum] = segments[LED_MINUS];
		if (ch == '.') {
			dispDigits[dispNum + 1] |= segments[LED_DOT];
			} else {
			if (dispNum > 0) dispNum--; else return;
		}
	}
}

void ledDispSymbol(uint8_t symbol, uint8_t position) {
	if (symbol < sizeof(segments) && position < sizeof(dispDigits)) dispDigits[position] = segments[symbol];
}

void ledDispRaw(uint8_t seg, uint8_t position) {
	if (position < sizeof(dispDigits)) dispDigits[position] = seg;
}

void ledClear() {
	memset(dispDigits, 0, sizeof(dispDigits));
}